package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.dto.GetAllForRelationDTO;
import com.diagbot.dto.IndexBatchDTO;
import com.diagbot.dto.KllisDetailDTO;
import com.diagbot.entity.KlConcept;
import com.diagbot.entity.KlDisease;
import com.diagbot.entity.TcmDisease;
import com.diagbot.entity.TcmSyndrome;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.LexiconEnum;
import com.diagbot.enums.StatusEnum;
import com.diagbot.service.impl.KlConceptServiceImpl;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.ConceptVO;
import com.diagbot.vo.KllisDetailVO;
import com.diagbot.vo.SearchConceptVO;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @author: gaodm
 * @time: 2021/1/26 13:05
 */
@Component
public class KlConceptFacade extends KlConceptServiceImpl {

    @Autowired
    private KlDiseaseFacade klDiseaseFacade;
    @Autowired
    private TcmDiseaseFacade tcmDiseaseFacade;
    @Autowired
    private TcmSyndromeFacade tcmSyndromeFacade;

    /**
     * 批量校验标准术语
     *
     * @param conceptVO
     * @return
     */
    public List<IndexBatchDTO> getConceptNames(ConceptVO conceptVO) {
        if (ListUtil.isEmpty(conceptVO.getNames())) {
            return new ArrayList<>();
        }
        List<IndexBatchDTO> retList = Lists.newLinkedList();
        //type： 类型：1-化验大项、2-化验小项、3-辅检、4-诊断、5-药品、6-手术和操作、7-科室、8-输血、10-量表、11-护理、12-中医诊断、13-中医证候
        Integer type = convertType(conceptVO.getType());

        QueryWrapper<KlConcept> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .in("lib_name", conceptVO.getNames());
        if (conceptVO.getType().equals(3)) {
            //辅检同时检索辅检项目和辅检子项
            queryWrapper.in("lib_type", Arrays.asList(LexiconEnum.PacsName.getKey(), LexiconEnum.PacsSubName.getKey()));
        } else {
            queryWrapper.eq("lib_type", type);
        }
        List<KlConcept> concepts = this.list(queryWrapper);

        if (ListUtil.isNotEmpty(concepts)) {
            for (KlConcept concept : concepts) {
                IndexBatchDTO dto = new IndexBatchDTO();
                dto.setId(concept.getId());
                dto.setName(concept.getLibName());
                retList.add(dto);
            }
        }

        if (ListUtil.isEmpty(retList)) {
            return retList;
        }
        List<Long> conceptIds = retList.stream().map(IndexBatchDTO::getId).collect(Collectors.toList());

        if (type.equals(LexiconEnum.Disease.getKey())) {
            List<KlDisease> diseases = klDiseaseFacade.list(new QueryWrapper<KlDisease>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .in("concept_id", conceptIds));
            if (ListUtil.isNotEmpty(diseases)) {
                Map<Long, KlDisease> idMap = diseases.stream().collect(Collectors.toMap(KlDisease::getConceptId, v -> v));
                for (IndexBatchDTO dto : retList) {
                    if (idMap.containsKey(dto.getId())) {
                        dto.setCode(idMap.get(dto.getId()).getIcd10Code());
                    }
                }
            }
        } else if (type.equals(LexiconEnum.Tcmdisease.getKey())) {
            List<TcmDisease> tcmDiseases = tcmDiseaseFacade.list(new QueryWrapper<TcmDisease>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .in("concept_id", conceptIds));
            if (ListUtil.isNotEmpty(tcmDiseases)) {
                Map<Long, TcmDisease> idMap = tcmDiseases.stream().collect(Collectors.toMap(TcmDisease::getConceptId, v -> v));
                for (IndexBatchDTO dto : retList) {
                    if (idMap.containsKey(dto.getId())) {
                        dto.setCode(idMap.get(dto.getId()).getCode());
                    }
                }
            }
        } else if (type.equals(LexiconEnum.Tcmsyndrome.getKey())) {
            List<TcmSyndrome> tcmSyndromes = tcmSyndromeFacade.list(new QueryWrapper<TcmSyndrome>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .in("concept_id", conceptIds));
            if (ListUtil.isNotEmpty(tcmSyndromes)) {
                Map<Long, TcmSyndrome> idMap = tcmSyndromes.stream().collect(Collectors.toMap(TcmSyndrome::getConceptId, v -> v));
                for (IndexBatchDTO dto : retList) {
                    if (idMap.containsKey(dto.getId())) {
                        dto.setCode(idMap.get(dto.getId()).getCode());
                    }
                }
            }
        }

        return retList;
    }


    public Integer convertType(Integer type) {
        Integer retType = null;
        //type： 类型：1-化验大项、2-化验小项、3-辅检、4-诊断、5-药品、6-手术和操作、7-科室、8-输血、10-量表、11-护理、12-中医诊断、13-中医证候
        switch (type) {
            case 1:
                retType = LexiconEnum.LisName.getKey();
                break;
            case 2:
                retType = LexiconEnum.LisSubName.getKey();
                break;
            case 3:
                retType = LexiconEnum.PacsName.getKey();
                break;
            case 4:
                retType = LexiconEnum.Disease.getKey();
                break;
            case 5:
                retType = LexiconEnum.Medicine.getKey();
                break;
            case 6:
                retType = LexiconEnum.Operation.getKey();
                break;
            case 7:
                retType = LexiconEnum.Dept.getKey();
                break;
            case 8:
                retType = LexiconEnum.Transfusion.getKey();
                break;
            case 10:
                retType = LexiconEnum.Scale.getKey();
                break;
            case 11:
                retType = LexiconEnum.Nurse.getKey();
                break;
            case 12:
                retType = LexiconEnum.Tcmdisease.getKey();
                break;
            case 13:
                retType = LexiconEnum.Tcmsyndrome.getKey();
                break;
            default:
                break;
        }
        return retType;
    }

    public List<GetAllForRelationDTO> searchConceptByNameAndLibType(SearchConceptVO searchConceptVO) {
        List<GetAllForRelationDTO> getAllForRelationDTOS = Lists.newArrayList();
        String name = searchConceptVO.getName();
        Integer libType = searchConceptVO.getLibType();
        List<Long> excludedConceptIds = searchConceptVO.getExcludedConceptIds();
        if (StringUtils.isNotBlank(name) && libType != null) {
            List<KlConcept> conceptList = this.list(new QueryWrapper<KlConcept>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .like(StringUtil.isNotBlank(name), "lib_name", name.trim())
                    .eq("lib_type", libType)
                    .eq("status", StatusEnum.Enable.getKey())
                    .notIn(ListUtil.isNotEmpty(excludedConceptIds), "id", excludedConceptIds));
            if (ListUtil.isNotEmpty(conceptList)) {
                getAllForRelationDTOS = conceptList.stream().map(x -> {
                    GetAllForRelationDTO getAllForRelationDTO = new GetAllForRelationDTO();
                    getAllForRelationDTO.setConceptNameType(x.getLibName());
                    getAllForRelationDTO.setConceptName(x.getLibName());
                    getAllForRelationDTO.setConceptId(x.getId());
                    getAllForRelationDTO.setLibType(x.getLibType());
                    return getAllForRelationDTO;
                }).collect(Collectors.toList());
            }
        }
        return getAllForRelationDTOS;
    }

    public Map<String, KllisDetailDTO> getLisDetaisByNamesFac(KllisDetailVO kllisDetailVO) {
        List<KllisDetailDTO> list = this.getLisDetaisByNames(kllisDetailVO);
        Map<String, KllisDetailDTO> res = EntityUtil.makeEntityMap(list, "libName");
        return res;
    }
}
